import 'package:logging/logging.dart';
import 'package:starting_template/core/constants/build_config_key.dart';
import 'package:starting_template/core/enums/data_source_type.dart';
import 'package:starting_template/core/enums/env_type.dart';
import 'package:starting_template/core/enums/feature_flag.dart';
import 'package:starting_template/core/logger/log_level.dart';

abstract class Env {
  /// Check if all **required** environment variables are defined, and throw
  /// an exception if not
  static void verify() {
    final List<BuildConfigKey> missingKeys = [];

    // Check required environment variables
    for (final key in BuildConfigKey.values) {
      if (key.isRequired && !key.isDefined) {
        missingKeys.add(key);
      }
    }

    if (missingKeys.isNotEmpty) {
      throw Exception(
        'Missing environment variables: '
        '${missingKeys.map((e) => e.key).join(', ')}',
      );
    }

    // Check optional environment variables
    // TODO(Wyatt): Check optional environment variables, for example:
    // if DataSource is API, then check for API base URL.
  }

  // REQUIRED ENVIRONMENT VARIABLES

  /// Returns environment type
  static EnvType get type => EnvType.fromString(BuildConfigKey.envName.get());

  /// Returns the data source type
  static DataSourceType get dataSource =>
      DataSourceType.fromString(BuildConfigKey.dataSource.get());

  // OPTIONAL ENVIRONMENT VARIABLES (can be null or have a default value)

  /// Returns the logger level
  static Level get loggerLevel =>
      LogLevel.fromString(BuildConfigKey.loggerLevel.getOr(() => 'info'));

  /// Returns the feature flags
  static List<FeatureFlag> get featureFlags {
    final flags = BuildConfigKey.featureFlags.getOr(() => '');
    if (flags.isEmpty) {
      return [];
    }

    return flags
        .split(',')
        .map(FeatureFlag.fromString)
        .whereType<FeatureFlag>()
        .toList();
  }

  // TODO(Wyatt): Add more environment variables here

  // HELPER METHODS

  /// Returns true if the environment is production
  static bool get isProduction => type == EnvType.prod;

  /// Returns true if the environment is staging
  static bool get isStaging => type == EnvType.staging;

  /// Returns true if the environment is development
  static bool get isDevelopment => type == EnvType.dev;

  /// Returns true if the data source is API
  static bool get dataSourceIsApi => dataSource == DataSourceType.api;

  /// Returns true if the data source is mock
  static bool get dataSourceIsMock => dataSource == DataSourceType.mock;

  /// Retourns true if a feature is enabled
  static bool isFeatureEnabled(FeatureFlag feature) =>
      featureFlags.contains(feature);

  /// Returns true if a feature is disabled
  static bool isFeatureDisabled(FeatureFlag feature) =>
      !isFeatureEnabled(feature);

  /// Returns true if a list of features is enabled
  static bool areFeaturesEnabled(List<FeatureFlag> features) {
    for (final feature in features) {
      if (!isFeatureEnabled(feature)) {
        return false;
      }
    }

    return true;
  }

  /// Returns true if a list of features is disabled
  static bool areFeaturesDisabled(List<FeatureFlag> features) {
    for (final feature in features) {
      if (isFeatureEnabled(feature)) {
        return false;
      }
    }

    return true;
  }

  /// Returns environment name
  static String get name => type.name;

  /// Returns environment summary
  static String get summary => BuildConfigKey.summary;
}
